

/**
 * Get a random color
 */
int randcol() {
  return irand(0,255) | (irand(0,255) << 8) | (irand(0,255) << 16);
}  

/**
 * Add a fixed amount (rate) to the RGB components of the color
 */
int fadecol(int c, int rate ) {
  
  int a = c & 0xff000000;
  int r = (c & 0xff0000)>>16;
  int g = (c & 0xff00)>>8;
  int b = c & 0xff;
  
  r += rate;
  g += rate;
  b += rate;
  if ( r < 0) r = 0;
  if ( g < 0) g = 0;
  if ( b < 0) b = 0;
  if ( r > 255) r = 255;
  if ( g > 255) g = 255;
  if ( b > 255) b = 255;

  return a | (r << 16) | (g << 8) | b;  
}


int darken(int c, int amt ) {
  int a = c & 0xff000000;
  int b = c & 0xff;
  int g = (c >> 8) & 0xff;
  int r = (c >> 16) & 0xff;
  r -= amt;
  g -= amt;
  b -= amt;
  if ( r < 0) r = 0;
  if ( g < 0) g = 0;
  if ( b < 0) b = 0;
  if ( r > 255) r = 255;
  if ( g > 255) g = 255;
  if ( b > 255) b = 255;
  return a | (r << 16) | (g<<8) | b;
}

int changeAlpha( int c, int newalpha ) {
  return (c & 0xffffff) | (newalpha << 24);
}


int darkenPerc(int c, float perc ) {
  int a = c & 0xff000000;
  int b = c & 0xff;
  int g = (c >> 8) & 0xff;
  int r = (c >> 16) & 0xff;
  r = round( 1.0 * r * perc);
  g = round( 1.0 * g * perc);
  b = round( 1.0 * b * perc);
  if ( r < 0) r = 0;
  if ( g < 0) g = 0;
  if ( b < 0) b = 0;
  if ( r > 255) r = 255;
  if ( g > 255) g = 255;
  if ( b > 255) b = 255;
  return a | (r << 16) | (g<<8) | b;
}


/**
 * Add a fixed amount (rate) to the RGB components of the color
 */
int fadecolToTarget(int c, int rate, int t ) {
  
  int a = c & 0xff000000;
  int r = (c & 0xff0000)>>16;
  int g = (c & 0xff00)>>8;
  int b = c & 0xff;

//  int ta = t & 0xff000000;
  int tr = (t & 0xff0000)>>16;
  int tg = (t & 0xff00)>>8;
  int tb = t & 0xff;

  if ( r > tr ) {
    r -= rate;
    if ( r < tr ) r = tr;
  } else {
    if ( r < tr ) {
      r += rate;
      if ( r > tr ) r = tr;      
    }
  }

  if ( g > tg ) {
    g -= rate;
    if ( g < tg ) g = tg;
  } else {
    if ( g < tg ) {
      g += rate;
      if ( g > tg ) g = tg;      
    }
  }

  if ( b > tb ) {
    b -= rate;
    if ( b < tb ) b = tb;
  } else {
    if ( b < tb ) {
      b += rate;
      if ( b > tb ) b = tb;      
    }
  }
  
  if ( r < 0) r = 0;
  if ( g < 0) g = 0;
  if ( b < 0) b = 0;
  if ( r > 255) r = 255;
  if ( g > 255) g = 255;
  if ( b > 255) b = 255;

  return a | (r << 16) | (g << 8) | b;  
}

int HSBtoRGB(int alpha, float hue, float saturation, float brightness) {
        int r = 0, g = 0, b = 0;
        if (saturation == 0) {
            r = g = b = (int) (brightness * 255.0f + 0.5f);
        } else {
            float h = (hue - (float)Math.floor(hue)) * 6.0f;
            float f = h - (float)java.lang.Math.floor(h);
            float p = brightness * (1.0f - saturation);
            float q = brightness * (1.0f - saturation * f);
            float t = brightness * (1.0f - (saturation * (1.0f - f)));
            switch ((int) h) {
            case 0:
                r = (int) (brightness * 255.0f + 0.5f);
                g = (int) (t * 255.0f + 0.5f);
                b = (int) (p * 255.0f + 0.5f);
                break;
            case 1:
                r = (int) (q * 255.0f + 0.5f);
                g = (int) (brightness * 255.0f + 0.5f);
                b = (int) (p * 255.0f + 0.5f);
                break;
            case 2:
                r = (int) (p * 255.0f + 0.5f);
                g = (int) (brightness * 255.0f + 0.5f);
                b = (int) (t * 255.0f + 0.5f);
                break;
            case 3:
                r = (int) (p * 255.0f + 0.5f);
                g = (int) (q * 255.0f + 0.5f);
                b = (int) (brightness * 255.0f + 0.5f);
                break;
            case 4:
                r = (int) (t * 255.0f + 0.5f);
                g = (int) (p * 255.0f + 0.5f);
                b = (int) (brightness * 255.0f + 0.5f);
                break;
            case 5:
                r = (int) (brightness * 255.0f + 0.5f);
                g = (int) (p * 255.0f + 0.5f);
                b = (int) (q * 255.0f + 0.5f);
                break;
            }
        }
        return (alpha << 24) | (r << 16) | (g << 8) | (b << 0);
    }
    
    
    
/**
 * Add a fixed amount (rate) to the RGB components of the color
 */
int fadeAlphaToTarget(int c, int rate, int t ) {
  
  int a = ((c & 0xff000000) >> 24) & 0xff;
  int r = (c & 0xff0000)>>16;
  int g = (c & 0xff00)>>8;
  int b = c & 0xff;

  int ta = ((t & 0xff000000) >> 24) & 0xff;

  if ( a > ta ) {
    a -= rate;
    if ( a < ta ) a = ta;
  } else {
    if ( a < ta ) {
      a += rate;
      if ( a > ta ) a = ta;      
    }
  }
  
  if ( a < 0) a = 0;
  if ( a > 255) a = 255;

  return (a << 24) | (r << 16) | (g << 8) | b;  
}    

// ----------------------------------------------------------------------    

void fadeCanvas( PGraphics gfx, int step, int targetcol) {
  int len = gfx.width * gfx.height; 
  for (int i = 0; i < len; i++ ) {
    gfx.pixels[i] = fadecolToTarget( gfx.pixels[i],  step, targetcol) ;    
  } 
}

// ----------------------------------------------------------------------    

void fadeCanvasAlpha( PGraphics gfx, int step, int targetalpha) {
  int len = gfx.width * gfx.height; 
  for (int i = 0; i < len; i++ ) {
//    gfx.pixels[i] = fadecolToTargetAlpha( gfx.pixels[i],  step, targetalpha) ;    
    gfx.pixels[i] = fadeAlphaToTarget( gfx.pixels[i],  step, targetalpha) ;
  } 
}
    
