/* ==================================================
 THE MUSIC OF PI
 https://www.algoritmarte.com/musicofpi/
 
 A simple visualization of the digits of Pi.
 ... highly NON optimized code :-) ...
 
 ==================================================    
 */


static int FPS = 20;

String SPIN; // here we load the digits of Pi
int plen;
int pidx;
int pcols[] = new int[11];
int currlev;

// colors
int precol[] = { 0x88987F, 0x4589C9, 0x70949E, 0x8423A4, 0x5DC3B6, 0x8291BA, 0x57988B, 0x8884B8, 0x565475, 0x407885 };
int useprecol = 0;  // set to 0 to use random colors

int MAXDIG = 1024*16;
PiDig[] diglist = new PiDig[ MAXDIG ];
int digidx;

boolean newworld;

// class for holding a Pi digit
class PiDig {
  int w;
  int h;
  int x, y;
  boolean filled;
  int lev;

  PiDig( int x, int y, int w, int h, int lev ) {
    this.x = x;
    this.y = y;
    this.w = w;
    this.h = h;
    this.lev = lev;
  }

  void split() {
    int nw = w / 2;
    int nh = h / 2;
    PiDig nd;
    nd = new PiDig( x, y, nw, nh, lev+1);
    diglist[ digidx++] = nd;
    nd = new PiDig( x + nw, y, w-nw, nh, lev+1);
    diglist[ digidx++] = nd;
    nd = new PiDig( x, y+nh, nw, h-nh, lev+1);
    diglist[ digidx++] = nd;
    nd = new PiDig( x+nw, y+nh, w-nw, h-nh, lev+1);
    diglist[ digidx++] = nd;
    if ( digidx > diglist.length ) digidx = 0;
  }

  void render( char c ) {
    noStroke();
    fill( pcols[ c - '0'] );
    rect( x, y, w, h );
    int fw = 2*h/3;
    textSize( fw );
    fill( 0xffffffff );
    textAlign( CENTER, CENTER );
    text( "" + c, x + w/2, y + h / 2 );
    filled = true;
    split();
  }
}

// ... prepare to launch ...
void setup() {
  size(1280, 720);
  background(0);
  frameRate( FPS );

  String ray[] = loadStrings( sketchPath("pi10.txt") );
  SPIN = ray[0];
  plen = SPIN.length();

  randWorld();
}

// ... init quadrants and randomize colors
void randWorld() {
  digidx = 0;
  PiDig d0 = new PiDig( 0, 0, 1280, 720, 0 );
  diglist[digidx++] = d0;
  d0.filled = true;
  d0.split();
  currlev = 1;

  print( "int precol[] = {" );
  for (int i = 0; i < 10; i++) {
    int c = randcol();
    if ( useprecol > 0) {
      c = precol[i];
    }
    print( ((i>0)? ",0x" : "0x") + hex(c) );
    pcols[i] = 0x70000000 | c;
  }
  println( "};" );
}

// press x to restart with new colors
void keyPressed() {
  if ( key == 'x' ) {
    newworld = true;
  }
}

// draw draw draw ... 
void draw() {

  if ( newworld ) {
    newworld = false;
    background(0);
    randWorld();
    pidx=0;
  }

  if ( (frameCount % (1*FPS/2) ) == 0 ) {
    
    char c = SPIN.charAt( pidx++ );
    if (c == '.') c = SPIN.charAt( pidx++ );

    int drawn = 0;
    while ( drawn == 0 ) {
      for (int i = 0; i < digidx; i++) {
        PiDig pd = diglist[i];
        if ( ! pd.filled && pd.lev == currlev ) {
          pd.render( c );
          drawn++;
          break;
        }
      }
      if ( drawn == 0) {
        currlev++;
      }
    }
  }
  saveToImage(); // see fSaveVideo and saveDir if you want to save the frames
}
